from sceneNode import SceneNode
from rectangle import Rectangle
from circle import Circle
from triangle import Triangle
from projectile import Projectile
from text import Text
from OpenGL.GLUT import *

from keys import keys

import math


class Defender(SceneNode):
    def __init__(self, parent, type = None, name = None):
	SceneNode.__init__(self,  parent, type, name)
	
	self.defaultVar("velocity", 180)
	self.defaultVar("altitude", 60)
	self.defaultVar("theta", 90)
	self.defaultVar("hitRadius", 5)
	self.defaultVar("moving", "stop")
	self.defaultVar("width", 20)
	self.defaultVar("length", 20)
	self.defaultVar("isFiring", False)
	self.defaultVar("lastFireTime", 0)
	
	self.defaultVar("ammo", 0)
	self.defaultVar("ammoRate", 5)
	self.defaultVar("maxAmmo", 50)
	
	self.addInitDependency("inputServer")
	self.addInitDependency("gfxServer")
	
    def init(self):
	inputServer = self.getFirstObjectOfName("inputServer")
	inputServer.addSubscriber(self)
	self.gfxServer = self.getFirstObjectOfName("gfxServer")
	
	self.screenWidth = self.gfxServer.getVar("screenWidth")
	self.screenHeight = self.gfxServer.getVar("screenHeight")
	
	r = self.getVar("altitude")
	theta = math.radians(self.getVar("theta"))
	self.setVar("position", (r*math.cos(float(theta))+(self.screenWidth/2), r*math.sin(float(theta))+(self.screenHeight/2), 0))
	
	self.gfx = Triangle(self)
	self.gfx.setVar("color", (0,1,0))
	
	self.ammoText = Text(self)
	self.ammoText.setVar("size", GLUT_BITMAP_HELVETICA_18)
	caption = "Ammo:  %i/%i" % (int(self.getVar("ammo")), self.getVar("maxAmmo"))
	self.ammoText.setVar("caption", caption)
	self.ammoText.setVar("position", (5, self.screenHeight - 25, 0))
	self.gfxServer.lateObjects.append(self.ammoText)
	
    def receiveMessage(self, cmd, msg = None):
	if cmd == "keyDown":
	    if msg == keys['left']:
		self.setVar("moving", "counter")
	    elif msg == keys["right"]:
		self.setVar("moving", "clock")
	    elif msg == keys["space"]:
		self.setVar("isFiring", True)
	elif cmd == "keyUp":
	    if msg == keys["left"]:
		self.setVar("moving", "stop")
	    elif msg == keys["right"]:
		self.setVar("moving", "stop")
	    elif msg == keys["space"]:
		self.setVar("isFiring", False)
	
    def update(self, timeTotal, timeDiff):
	direction = self.getVar("moving")
	r = self.getVar("altitude")
	theta = self.getVar("theta")
	velocity = self.getVar("velocity")
	if direction == "clock":
	    theta = theta - velocity*timeDiff
	    self.setVar("theta", theta)
	    theta = math.radians(theta)
	    self.setVar("position", (r*math.cos(float(theta))+(self.screenWidth/2), r*math.sin(float(theta))+(self.screenHeight/2), 0))
	elif direction == "counter":
	    theta = theta + velocity*timeDiff
	    self.setVar("theta", theta)
	    theta = math.radians(theta)
	    self.setVar("position", (r*math.cos(float(theta))+(self.screenWidth/2), r*math.sin(float(theta))+(self.screenHeight/2), 0))
	theta = self.getVar("theta")
	if theta > 360:
	    theta = theta - 360
	    self.setVar("theta", theta)
	elif theta < 0:
	    theta = theta + 360
	    self.setVar("theta", theta)
	self.setVar("rotation",(0,0,self.getVar("theta")-90))
	
	self.setVar("scale", (self.getVar("width"), self.getVar("length"), 1))
	
	ammo = self.getVar("ammo")
	ammoRate = self.getVar("ammoRate")
	maxAmmo = self.getVar("maxAmmo")
	ammo += ammoRate*timeDiff
	if (ammo < maxAmmo):
	    self.setVar("ammo", ammo)
	caption = "Ammo:  %i/%i" % (int(self.getVar("ammo")), self.getVar("maxAmmo"))
	self.ammoText.setVar("caption", caption)
	
	if self.getVar("isFiring") == True:
	    if timeTotal - self.getVar("lastFireTime") > 0.05 and int(self.getVar("ammo")) > 0:
		self.setVar("lastFireTime", timeTotal)
	        projectiles = self.getFirstObjectOfName("projectiles")
	        projectile = Projectile(projectiles)
	        projectile.setVar("position", self.getVar("position"))
	        projectile.setVar("theta", self.getVar("theta"))
		self.setVar("ammo", self.getVar("ammo") - 1)
