import pygame, sys

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

from object import Object
from sceneNode import SceneNode
from rectangle import Rectangle
from circle import Circle
from triangle import Triangle

import math

class GameGfxServer(Object):

    def __init__(self, parent):
        Object.__init__(self, parent, name = "gfxServer")
	
	self.defaultVar("screenWidth", 600)
	self.defaultVar("screenHeight", 600)
	self.defaultVar("windowTitle", "Orbital Defender")
	self.defaultVar("renderDimensions", 2)
	self.defaultVar("camerax", 0)
	self.defaultVar("cameray", 0)
	
	self.textureList = {}
	self.earlyObjects = []
	self.lateObjects = []
	self.explosionObjects = []
	
    # Initialize the game window
    def init(self):
	pygame.init()
	glutInit(sys.argv)
	
	screenWidth = self.getVar("screenWidth")
	screenHeight = self.getVar("screenHeight")
	size = screenWidth, screenHeight
	
	self.screen = pygame.display.set_mode((screenWidth, screenHeight), pygame.OPENGLBLIT | pygame.DOUBLEBUF | pygame.HWSURFACE)
	
	pygame.display.set_caption(self.getVar("windowTitle"))
	
	glLoadIdentity()
        glMatrixMode(GL_PROJECTION)
	gluOrtho2D(0,screenWidth,0,screenHeight)
	
	glEnable(GL_DEPTH_TEST)
	glEnable(GL_TEXTURE_2D)
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
	glClearColor(0, 0, 0, 0)
	
    def preRender(self):
        glMatrixMode(GL_MODELVIEW)
        glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT)
        glLoadIdentity()
	
#	glTranslate(self.getVar("screenWidth") / 2, self.getVar("screenHeight") / 2, 0)
	
	self.renderCount = 0
	
	for object in self.earlyObjects:
	    if object.getVar("render"):
	        self.renderObject(object)
    
    def render(self, object):
	if object.getVar("render") and object not in self.earlyObjects and object not in self.lateObjects:
	    if issubclass(object.__class__,SceneNode):
		self.renderObject(object)
	    else:
		for child in object.children.itervalues():
		    self.render(child)
    
    def postRender(self):
	for object in self.lateObjects:
	    if object.getVar("render"):
	        self.renderObject(object)
	for object in self.explosionObjects:
	    if object.getVar("render"):
	        self.renderObject(object)
	
	glFlush()
	pygame.display.flip()
    
    def renderObject(self, object):
	glPushMatrix()
	glPushAttrib(GL_ALL_ATTRIB_BITS)
	
	glTranslate(0,0,0.0001 * self.renderCount)
        self.renderCount += 1
	
	screenWidth = self.getVar("screenWidth")
	screenHeight = self.getVar("screenHeight")
	
	sx,sy,sz = object.getVar("scale")
	x,y,z = object.getVar("position")
        glTranslatef(x-(sx/2), y-(sy/2), z)
	
	yaw,pitch,roll = object.getVar("rotation")
        glRotatef(yaw,0,1,0)
        glRotatef(pitch,1,0,0)
        glRotatef(roll,0,0,1)
	
        glScalef(sx,sy,sz)
	
	if object.hasVar("color"):
            r,g,b = object.getVar("color")
            glColor3f(r,g,b)
	
	if object.hasVar("texture"):
	    self.bindTexture(object.getVar("texture"))
	else:
	    self.unbindTexture()
	
	name = "render" + object.__class__.__name__
	if hasattr(self,name) and callable(getattr(self,name)):
	    getattr(self,name)(object)
	    
	for child in object.children.itervalues():
	    self.renderObject(child)
	
	glPopAttrib()
	glPopMatrix()

    def renderRectangle(self, object):
	glBegin(GL_QUADS)
        glTexCoord2f(0,0)
        glVertex3f(0,0,0)
        glTexCoord2f(0,1)
        glVertex3f(0,1,0)
        glTexCoord2f(1,1)
        glVertex3f(1,1,0)
        glTexCoord2f(1,0)
        glVertex3f(1,0,0)
        glEnd()
	
#        glBegin(GL_QUADS)
#	glTexCoord2f(-0.5, -0.5)
#	glVertex3f(-0.5, -0.5, 0)
	
#	glTexCoord2f(-0.5, 0.5)
#        glVertex3f(-0.5, 0.5, 0)
	
#	glTexCoord2f(0.5, 0.5)
#        glVertex3f(0.5, 0.5, 0)
	
#	glTexCoord2f(0.5, -0.5)
#        glVertex3f(0.5, -0.5, 0)
#        glEnd()
	
    def renderCircle(self, object):
	r = 0.5
	n = object.getVar("numSections")
	glBegin(GL_TRIANGLE_FAN)
	for i in range(n):
	    theta = i * (2 * math.pi / n)
	    glVertex3f(r * math.cos(theta), r * math.sin(theta), 0)
	glEnd()
	
    def renderTriangle(self, object):
	glBegin(GL_TRIANGLES)
	glVertex3f(-0.5,-0.5,0)
	glVertex3f(0.5,-0.5,0)
	glVertex3f(0,0.5,0)
	glEnd()
	
    def renderText(self, object):
#	font = pygame.font.Font(None, 36)
#	text = font.render("Testing", 1, (255,255,255))
#	self.screen.blit(text, (200,200))
	caption = object.getVar("caption")
	x,y,z = object.getVar("position")
	glPushMatrix()
	i = 0
	for line in caption.split('\n'):
	    glRasterPos(0, -20*i, 0)
	    for char in line:
		glutBitmapCharacter(object.getVar("size"),ord(char))
	    i += 1
	glPopMatrix()

    def loadTexture(self, filename):
	texture = pygame.image.load(filename)
	w,h = texture.get_size()
	texture_gl = pygame.image.tostring(texture, "RGBA", 1)
	texture_id = glGenTextures(1)
	glBindTexture(GL_TEXTURE_2D, texture_id)
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP)
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP)
	
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,w,h,0,GL_RGBA,GL_UNSIGNED_BYTE,texture_gl)
	
	return texture_id
	
    def bindTexture(self, filename):
	if not self.textureList.has_key(filename):
	    self.textureList[filename] = self.loadTexture(filename)
	texture = self.textureList[filename]
	glBindTexture(GL_TEXTURE_2D, texture)
	
    def unbindTexture(self):
	glBindTexture(GL_TEXTURE_2D, -1)
