class Object:

    def __init__(self, parent, type = None, name = None):
	self.name = name
	self.parent = parent
	self.type = type
	
	self.vars = {}
	self.children = {}
	
	self.subscribers = []
	self.initDependencies = []
	self.childrenToRemove = []
	
	self.initialized = False
	self.destroyed = False
	self.timeToLive = None
	
	if not self.type:
	    className = self.__class__.__name__
	    self.type = className[:1].lower() + className[1:]
	
	# Handle no name classes or duplicate name classes
	if not self.name or self.parent.hasChild(self.name):
	    if self.parent:
	        self.name = self.type
		tag = 0
		while self.parent.hasChild(self.name):
		    self.name = "%s_%d" % (self.type, tag)
		    tag = tag + 1
	    else:
	        self.name = "root"
	
	# Add children to parents and set path
	if self.parent:
	    # Add object name to children list of parent
	    self.parent.children[self.name] = self
	    if self.parent.parent:
	        self.path = self.parent.path + '/' + self.name
	    else:
	        self.path = '/' + self.name
	else:
	    self.path = '/'
	    
	self.defaultVar("render", True)
	
	if self.parent and self.parent.initialized:
	    self.preInit()
	    
#	print "INIT  " , self.path

    # Returns whether or not the current object has the
    # variable 'var' in it's variable list
    def hasVar(self, var):
        return self.vars.has_key(var)

    # Defaults the variable 'name' to 'value'
    def defaultVar (self, name, value):
	if not self.hasVar(name):
	    self.setVar(name,value)

    # Sets the value of the variable 'name' to 'val'
    def setVar (self, name, val):
	self.vars[name] = val

    # Returns the value of the variable named 'name'
    def getVar (self, name):
	if self.vars.has_key(name):
	    return self.vars[name]
	else:
	    return None
	
    # Returns the root of the tree through recursive function
    # calls to an object's parent until the object has no parent.
    # An object with no parent is the root of the tree
    def getRoot(self):
        if (self.parent):
	    return self.parent.getRoot()
	else:
	    return self
	
    # Returns the parent to the current object.  If the object has 
    # no parent, then it is the root and nothing is returned
    def getParent(self):
        if (self.parent):
	    return self.parent
	else:
	    return None
	
    # Returns the first object with the specified name.  Performs
    # a depth first search on the tree and returns the object
    # when found.  If not found, returns None
    def getFirstObjectOfName(self, objName, top = True):
        if top:
	    return self.getRoot().getFirstObjectOfName(objName, False)
	elif self.name == objName:
	    return self
	else:
	    for child in self.children.itervalues():
	        object = child.getFirstObjectOfName(objName, False)
		if object:
		    return object
	    return None
	
    # Adds the value of the variable "subscriber" to the
    # list of subscribers to the current object.  The contents
    # of the variable "subscriber" is a path to an object
    def addSubscriber(self, subscriber):
        if not subscriber.__class__ == str:
	    subscriber = subscriber.path
        self.subscribers.append(subscriber)
#	print "Adding subscriber" , subscriber , " to " , self.path
    
    # Returns whether the current has the child "childName"
    # in the list of children
    def hasChild(self, childName):
        return self.children.has_key(childName)
    
    # Handles any tasks that must be completed before the
    # actual object is updated
    def preUpdate(self, timeTotal, timeDiff):
	if self.initialized and not self.destroyed:
	    if self.timeToLive:
		if self.timeToLive < 0:
		    self.delete()
		else:
		    self.timeToLive -= timeDiff
		
	    for name in self.childrenToRemove:
	        if self.children.has_key(name):
		    child = self.children.pop(name)
		    child.preDestroy()
	    self.childrenToRemove = []
	
            self.update(timeTotal, timeDiff)
	
	    for child in self.children.itervalues():
	        child.preUpdate(timeTotal, timeDiff)
	
    # Place holder in the inheritance tree.  Any object
    # that has special update instructions will have its
    # own update function
    def update(self, timeTotal, timeDiff):
#	print "UPDATE   " , self.path
        pass
    
    # Publishes the given data to all subscribers to the
    # object.  Done by grabbing each object in the
    # subscribers list and calling the receiveMessage
    # function on that object
    def publish(self, cmd, msg):
        for subscriber in self.subscribers:
	    #print self.path , " publishing cmd: " , cmd , "   msg: " , msg , "    to object: " , subscriber
	    self.getObject(subscriber).receiveMessage(cmd, msg)
    
    # Returns the object of the given path.  If the object
    # doesn't exist, the function returns None
    def getObject(self, path):
        if path[0] == '/':
	    if len(path) > 1:
	        return self.getRoot().getObject(path[1:])
	    else:
	        return self.getRoot()
	elif path.find('/') != -1:
	    lead = path.split('/')[0]
	    rest = path[len(lead)+1:]
	    if lead == '..':
	        return self.getParent().getObject(rest)
	    elif self.hasChild(lead):
	        return self.getChild(lead).getObject(rest)
	    else:
		return None
	else:
	    if path == '..':
		return self.getParent()
	    elif self.hasChild(path):
		return self.getChild(path)
	    else:
		return None
	    
    # Returns the specified child name if the current object
    # has that child
    def getChild(self, child):
	if self.hasChild(child):
	    return self.children[child]
	else:
	    return None
    # Place holder in the inheritance tree.  Any object
    # that handles messages has its own function	
    def receiveMessage(self, cmd, msg = None):
	pass
    
    # Handles all per-initialization stuff when an object is
    # initialized.  Makes sure that all objects that need
    # to be initialized before the current object are
    # initialized, then initializes the current object,
    # then initializes all of the children
    def preInit(self):
	if not self.initialized:
	    for path in self.initDependencies:
	        obj = self.getObject(path)
	        if (obj):
	            obj.preInit()
	    self.init()
	    for child in self.children.itervalues():
	        child.preInit()
	    self.initialized = True
	
    # Place holder in the inheritance tree.  Any object that
    # needs special initialization has its own init function
    def init(self):
	pass
    
    # Returns whether there is an object(s) that has to be
    # initialized before the current object
    def hasInitDependency(self, dependPath):
	return dependPath in self.initDependencies
    
    # Adds the specified path to the list of objects that
    # must be initialized before the current object
    def addInitDependency(self, dependPath):
	if not self.hasInitDependency(dependPath):
	    self.initDependencies.append(dependPath)
	
    # First in a series of functions to delete the current
    # object.  Appends object name to the parent object's
    # list of children to be deleted
    def delete(self):
	if self.parent:
	    self.parent.childrenToRemove.append(self.name)
	
    def preDestroy(self):
	if self.initialized and not self.destroyed:
	    self.destroyed = True
	    
	    gfxServer = self.getFirstObjectOfName("gfxServer")
	    if self in gfxServer.earlyObjects:
		gfxServer.earlyObjects.remove(self)
	    if self in gfxServer.lateObjects:
		gfxServer.lateObjects.remove(self)
	    if self in gfxServer.explosionObjects:
		gfxServer.explosionObjects.remove(self)
		
	    self.destroy()
	    
	    for child in self.children.itervalues():
		child.preDestroy()
	
    def destroy(self):
	pass
#	print "DELETE  " , self.path