from sceneNode import SceneNode
from circle import Circle
from rectangle import Rectangle

import math

class Projectile(SceneNode):
    def __init__(self, parent, type = None, name = None):
	SceneNode.__init__(self,  parent, type, name)
	
	self.defaultVar("velocity", 300)
	self.defaultVar("theta", 0)
	self.defaultVar("hitRadius", 5)
	
	hr = self.getVar("hitRadius")
	self.setVar("scale", (hr*2, hr*2, hr*2))
	self.setVar("color", (1,0,1))
	
	self.gfx = Circle(self)
	self.gfx.setVar("color", (1,0,1))
	
    def init(self):
	self.invaderManager = self.getFirstObjectOfName("invaderManager")
	self.soundServer = self.getFirstObjectOfName("soundServer")
	self.addSubscriber(self.soundServer)
	
	
    def update(self, timeTotal, timeDiff):
	x,y,z = self.getVar("position")
	theta = math.radians(self.getVar("theta"))
	velocity = self.getVar("velocity")
	x = x + (velocity * math.cos(theta)) * timeDiff
	y = y + (velocity * math.sin(theta)) * timeDiff
	self.setVar("position", (x,y,z))
	self.setVar("rotation", (0,0,theta))
	
	gfxServer = self.getFirstObjectOfName("gfxServer")
	screenWidth = gfxServer.getVar("screenWidth")
	screenHeight = gfxServer.getVar("screenHeight")
	if x < 0 or x > screenWidth or y < 0 or y > screenHeight:
	    self.delete()
	
	for invader in self.invaderManager.children.itervalues():
	    if invader.initialized and not invader.destroyed and not invader.getVar("dead"):
	        x1,y1,z1 = self.getVar("position")
	        x2,y2,z2 = invader.getVar("position")
	        r1 = self.getVar("hitRadius")
	        r2 = invader.getVar("hitRadius")
	        dist = math.sqrt( (x2-x1)**2 + (y2-y1)**2 + (z2-z1)**2 )
	        if dist < r1 + r2:
		    numInvaders = self.invaderManager.getVar("numCurrentInvaders")
		    self.publish("playSound", "missileCollision")
		    self.invaderManager.setVar("numCurrentInvaders", numInvaders-1)
		    invader.setVar("dead", True)
		    invader.delete()
		    self.delete()
